<?php

/**
 * ESTES Carrier Services
 *
 * @package     ESTES Freight Quotes
 * @author      Eniture-Technology
 */
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Get Estes Freight Quotes Rate Class
 */
class Estes_get_shipping_quotes extends Estes_Quotes_Liftgate_As_Option
{

    /**
     * Create Shipping Package
     * @param $packages
     * @return array
     */
    public $en_wd_origin_array;
    public $quote_settings;
    public $localdeliver;

    function __construct()
    {
        $this->quote_settings = array();
    }

    function estes_shipping_array($packages, $package_plugin = "")
    {
        // FDO
        $EnEstesFdo = new EnEstesFdo();
        $en_fdo_meta_data = array();
        // check plan for nested material
        $nested_plan = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'nested_material');
        $nestingPercentage = $nestedDimension = $nestedItems = $stakingProperty = [];
        $doNesting = false;
        $insurance_applied = '0';
        $product_markup_shipment = 0;

        foreach ($packages['items'] as $item) {
            $product_name[] = $item['productName'];
            isset($item['insurance']) && $item['insurance'] == 'yes' ? $insurance_applied = '1' : '0';
            isset($item['nestedMaterial']) && !empty($item['nestedMaterial']) &&
            $item['nestedMaterial'] == 'yes' && !is_array($nested_plan) ? $doNesting = 1 : "";

            if (!empty($item['markup']) && is_numeric($item['markup'])) {
                $product_markup_shipment += $item['markup'];
            }
        }
        $estes_woo_obj = new ESTES_Woo_Update_Changes();
        $destinationAddress = $this->destinationAddressEstes();
        $residential_detecion_flag = get_option("en_woo_addons_auto_residential_detecion_flag");

        $accessorials = array_filter(
            array(
                ((get_option('estes_settings_liftgate') === 'yes') ? $estes_liftgate = 'LGATE' : ''),
                ((get_option('estes_setting_residential') === 'yes') ? $estes_residential = 'HD' : ''),
                ((get_option('estes_ltl_quotes_liftgate_delivery_as_option') === 'yes') ? $estes_liftgate = 'LGATE' : '')
            )
        );

        // White Glove
        $accessorials = apply_filters('en_wg_accessorials', $accessorials);

        $this->en_wd_origin_array = (isset($packages['origin'])) ? $packages['origin'] : array();
        $address = $packages['origin']['city'] . ", " . $packages['origin']['state'] . ", " . $packages['origin']['zip'];
        $domain = estes_freight_get_domain();

        // Cuttoff Time
        $shipment_week_days = "";
        $order_cut_off_time = "";
        $shipment_off_set_days = "";
        $modify_shipment_date_time = "";
        $store_date_time = "";
        $estes_delivery_estimates = get_option('estes_delivery_estimates');
        $shipment_week_days = $this->estes_shipment_week_days();
        if ($estes_delivery_estimates == 'delivery_days' || $estes_delivery_estimates == 'delivery_date') {
            $order_cut_off_time = $this->quote_settings['orderCutoffTime'];
            $shipment_off_set_days = $this->quote_settings['shipmentOffsetDays'];
            $modify_shipment_date_time = ($order_cut_off_time != '' || $shipment_off_set_days != '' || (is_array($shipment_week_days) && count($shipment_week_days) > 0)) ? 1 : 0;
            $store_date_time = $today = date('Y-m-d H:i:s', current_time('timestamp'));
        }

        // FDO
        $en_fdo_meta_data = $EnEstesFdo->en_cart_package($packages);

        // Version numbers
        $plugin_versions = $this->en_version_numbers();

        $post_data = array(
            // Version numbers
            'plugin_version' => $plugin_versions["en_current_plugin_version"],
            'wordpress_version' => get_bloginfo('version'),
            'woocommerce_version' => $plugin_versions["woocommerce_plugin_version"],
            'licence_key' => get_option('estes_setting_licnse_key'),
            'requestKey' => md5(microtime() . rand()),
            'sever_name' => $this->estes_parse_url($domain),
            'carrierName' => 'estes',
            'carrier_mode' => 'pro',
            'version' => '2.0',
            'plateform' => 'WordPress',
            'UserName' => get_option('estes_setting_username'),
            'Password' => get_option('estes_setting_password'),
            'CUSTNMBR' => get_option('estes_setting_customer_no'),
            'product_name' => $product_name,
            'sender_origin' => $packages['origin']['location'] . ": " . $packages['origin']['city'] . ", " . $packages['origin']['state'] . " " . $packages['origin']['zip'],
            'suspend_residential' => get_option('suspend_automatic_detection_of_residential_addresses'),
            'residential_detecion_flag' => $residential_detecion_flag,
            'Role' => 'S',
            'PaymentType' => 'P',
            'senderCity' => $packages['origin']['city'],
            'senderState' => $packages['origin']['state'],
            'senderZip' => $packages['origin']['zip'],
            'senderCountryCode' => $this->estes_get_country_code($packages['origin']['country']),
            'receiverCity' => (isset($destinationAddress['city'])) ? $destinationAddress['city'] : 'N',
            'receiverState' => $destinationAddress['state'],
            'receiverZip' => str_replace(" ", "", trim($destinationAddress['zip'])),
            'receiverCountryCode' => $this->estes_get_country_code($destinationAddress['country']),
            'accessorials' => $accessorials,
            'shipmentDate' => date('Y-m-d'),
            'handlingUnitWeight' => get_option('estes_freight_handling_weight'),
            'maxWeightPerHandlingUnit' => get_option('estes_freight_maximum_handling_weight'),
            'commdityDetails' => array(
                'handlingUnitDetails' => array(
                    'wsHandlingUnit' => $this->estes_get_line_items($packages),
                ),
            ),
            // FDO
            'en_fdo_meta_data' => $en_fdo_meta_data,
            // Nesting
            'doNesting' => $doNesting,
            // Cuttoff Time
            'modifyShipmentDateTime' => $modify_shipment_date_time,
            'OrderCutoffTime' => $order_cut_off_time,
            'shipmentOffsetDays' => $shipment_off_set_days,
            'storeDateTime' => $store_date_time,
            'shipmentWeekDays' => $shipment_week_days,
            'origin_markup' => (isset($packages['origin']['origin_markup'])) ? $packages['origin']['origin_markup'] : 0,
            'product_level_markup' => $product_markup_shipment
        );

        // White Glove
        $post_data = apply_filters('en_wg_rad', $post_data);

        // Hold At Terminal
        $hold_at_terminal = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'estes_freight_hold_at_terminal');
        if (!is_array($hold_at_terminal)) {
            (isset($this->quote_settings['HAT_status']) && ($this->quote_settings['HAT_status'] == 'yes')) ? $post_data['holdAtTerminal'] = '1' : '';
        }
        // Insurance Fee
        $action_insurance = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'insurance_fee');
        if (!is_array($action_insurance)) {
            $post_data['addInsurance'] = $insurance_applied;
        }
        // Hazardous Material
        $hazardous_material = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'hazardous_material');
        if (!is_array($hazardous_material)) {
            (isset($packages['hazardous_material'])) ? $post_data['accessorials'][] = 'HAZ' : "";
            (isset($packages['hazardous_material']) == 'yes') ? $post_data['hazardous'][] = 'H' : '';
            // FDO
            $post_data['en_fdo_meta_data'] = array_merge($post_data['en_fdo_meta_data'], $EnEstesFdo->en_package_hazardous($packages, $en_fdo_meta_data));
        }

        // In-store pickup and local delivery
        $instore_pickup_local_devlivery_action = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'instore_pickup_local_devlivery');
        if (!is_array($instore_pickup_local_devlivery_action)) {
            $post_data['InstorPickupLocalDelivery'] = apply_filters('en_estes_wd_standard_plans', $post_data, $post_data['receiverZip'], $this->en_wd_origin_array, $package_plugin);
        }

        $post_data = $this->estes_quotes_update_carrier_service($post_data);
        $post_data = apply_filters("en_woo_addons_carrier_service_quotes_request", $post_data, en_woo_plugin_estes_ltl_quotes);
        // Configure standard plugin with pallet packaging addon
        $post_data = apply_filters('en_pallet_identify', $post_data);

        do_action("eniture_debug_mood", "Quotes Request (ltl)", $post_data);
        do_action("eniture_debug_mood", "Plugin Features (estes ltl)", get_option('eniture_plugin_13'));

        return $post_data;
    }

    /**
     * Return version numbers
     * @return int
     */
    function en_version_numbers()
    {
        if (!function_exists('get_plugins'))
            require_once(ABSPATH . 'wp-admin/includes/plugin.php');

        $plugin_folder = get_plugins('/' . 'woocommerce');
        $plugin_file = 'woocommerce.php';
        $wc_plugin = (isset($plugin_folder[$plugin_file]['Version'])) ? $plugin_folder[$plugin_file]['Version'] : "";
        $get_plugin_data = get_plugin_data(ESTES_MAIN_FILE);
        $plugin_version = (isset($get_plugin_data['Version'])) ? $get_plugin_data['Version'] : '';

        $versions = array(
            "woocommerce_plugin_version" => $wc_plugin,
            "en_current_plugin_version" => $plugin_version
        );

        return $versions;
    }

    /**
     * @return shipment days of a week  - Cuttoff time
     */
    public function estes_shipment_week_days()
    {
        $shipment_days_of_week = array();

        if (get_option('all_shipment_days_estes') == 'yes') {
            return $shipment_days_of_week;
        }

        if (get_option('monday_shipment_day_estes') == 'yes') {
            $shipment_days_of_week[] = 1;
        }
        if (get_option('tuesday_shipment_day_estes') == 'yes') {
            $shipment_days_of_week[] = 2;
        }
        if (get_option('wednesday_shipment_day_estes') == 'yes') {
            $shipment_days_of_week[] = 3;
        }
        if (get_option('thursday_shipment_day_estes') == 'yes') {
            $shipment_days_of_week[] = 4;
        }
        if (get_option('friday_shipment_day_estes') == 'yes') {
            $shipment_days_of_week[] = 5;
        }

        return $shipment_days_of_week;
    }

    /**
     * Estes Line Items
     * @param type $packages
     * @return Array Line Items
     */
    function estes_get_line_items($packages)
    {
        $lineItem = array();
        foreach ($packages['items'] as $item) {

            // Standard Packaging
            $ship_as_own_pallet = isset($item['ship_as_own_pallet']) && $item['ship_as_own_pallet'] == 'yes' ? 1 : 0;
            $vertical_rotation_for_pallet = isset($item['vertical_rotation_for_pallet']) && $item['vertical_rotation_for_pallet'] == 'yes' ? 1 : 0;
            $counter = $estes_counter = (isset($item['variantId']) && $item['variantId'] > 0) ? $item['variantId'] : $item['productId'];
            $lineItem[$estes_counter] = array(
                'lineItemDetails' => array(
                    'wsLineItem' => array($estes_counter =>
                        array(
                            'lineItemClass' => $item['productClass'],
                            'lineItemWeight' => $item['productWeight'],
                            'lineItemPieceType' => 'PL',
                            'piecesOfLineItem' => $item['productQty'],
                            'lineItemLength' => $item['productLength'],
                            'lineItemWidth' => $item['productWidth'],
                            'lineItemHeight' => $item['productHeight'],
                            'lineItemPrice' => $item['productPrice'],
                            // Nested indexes
                            'nestingPercentage' => $item['nestedPercentage'],
                            'nestingDimension' => $item['nestedDimension'],
                            'nestedLimit' => $item['nestedItems'],
                            'nestedStackProperty' => $item['stakingProperty'],
                            // Shippable handling units
                            'lineItemPalletFlag' => $item['lineItemPalletFlag'],
                            'lineItemPackageType' => $item['lineItemPackageType'],

                            // Standard Packaging
                            'shipPalletAlone' => $ship_as_own_pallet,
                            'vertical_rotation' => $vertical_rotation_for_pallet
                        ),
                    ),
                ),
            );

            $lineItem[$counter] = apply_filters('en_fdo_carrier_service', $lineItem[$counter], $item);
        }
        return $lineItem;
    }

    /**
     * Get Country Code
     * @param $sCountryCode
     * @return string
     */
    function estes_get_country_code($sCountryCode)
    {
        switch (trim($sCountryCode)) {
            case 'CN':
                $sCountryCode = "CN";
                break;
            case 'CAN':
                $sCountryCode = "CN";
                break;
            case 'CA':
                $sCountryCode = "CN";
                break;
            case 'US':
                $sCountryCode = "US";
                break;
            case 'USA':
                $sCountryCode = "US";
                break;
        }
        return $sCountryCode;
    }

    function destinationAddressEstes()
    {
        $en_order_accessories = apply_filters('en_order_accessories', []);
        if (isset($en_order_accessories) && !empty($en_order_accessories)) {
            return $en_order_accessories;
        }

        $estes_woo_obj = new ESTES_Woo_Update_Changes();
        $freight_zipcode = (strlen(WC()->customer->get_shipping_postcode()) > 0) ? WC()->customer->get_shipping_postcode() : $estes_woo_obj->estes_postcode();
        $freight_state = (strlen(WC()->customer->get_shipping_state()) > 0) ? WC()->customer->get_shipping_state() : $estes_woo_obj->estes_getState();
        $freight_country = (strlen(WC()->customer->get_shipping_country()) > 0) ? WC()->customer->get_shipping_country() : $estes_woo_obj->estes_getCountry();
        $freight_city = (strlen(WC()->customer->get_shipping_city()) > 0) ? WC()->customer->get_shipping_city() : $estes_woo_obj->estes_getCity();
        return array(
            'city' => $freight_city,
            'state' => $freight_state,
            'zip' => $freight_zipcode,
            'country' => $freight_country
        );
    }

    /**
     * Get Nearest Address If Multiple Warehouses
     * @param $warehous_list
     * @param $receiverZipCode
     * @return array
     */
    function estes_multi_warehouse($warehous_list, $receiverZipCode)
    {
        if (count($warehous_list) == 1) {
            $warehous_list = reset($warehous_list);
            return $this->estes_origin_array($warehous_list);
        }

        $estes_distance_request = new Get_estes_freight_distance();
        $accessLevel = "MultiDistance";
        $response_json = $estes_distance_request->estes_freight_address($warehous_list, $accessLevel, $this->destinationAddressEstes());

        $response_json = json_decode($response_json);
        $min_dist_origin = isset($response_json->origin_with_min_dist) ? $response_json->origin_with_min_dist : '';
        return $this->estes_origin_array($min_dist_origin);
    }

    /**
     * Create Origin Array
     * @param $origin
     * @return array
     */
    function estes_origin_array($origin)
    {
        // In-store pickup and local delivery
        if (has_filter("en_estes_wd_origin_array_set")) {
            return apply_filters("en_estes_wd_origin_array_set", $origin);
        }

        return array('locationId' => $origin->id, 'zip' => $origin->zip, 'city' => $origin->city, 'state' => $origin->state, 'location' => $origin->location, 'country' => $origin->country, 'origin_markup' => $origin->origin_markup);
    }

    /**
     * Refine URL
     * @param $domain
     * @return url
     */
    function estes_parse_url($domain)
    {
        $domain = trim($domain);
        $parsed = parse_url($domain);
        if (empty($parsed['scheme'])) {
            $domain = 'http://' . ltrim($domain, '/');
        }
        $parse = parse_url($domain);
        $refinded_domain_name = $parse['host'];
        $domain_array = explode('.', $refinded_domain_name);
        if (in_array('www', $domain_array)) {
            $key = array_search('www', $domain_array);
            unset($domain_array[$key]);
            if(phpversion() < 8) {
                $refinded_domain_name = implode($domain_array, '.'); 
            }else {
                $refinded_domain_name = implode('.', $domain_array);
            }
        }
        return $refinded_domain_name;
    }

    function return_estes_localdelivery_array()
    {
        return $this->localdeliver;
    }

    /**
     * Curl Request To Get Quotes
     * @param $request_data
     */
    function estes_get_web_quotes($request_data)
    {
        // Check response from session
        $srequest_data = $request_data;
        $srequest_data['requestKey'] = "";
        $currentData = md5(json_encode($srequest_data));
        $requestFromSession = WC()->session->get('previousRequestData');
        $requestFromSession = ((is_array($requestFromSession)) && (!empty($requestFromSession))) ? $requestFromSession : array();

        if (isset($requestFromSession[$currentData]) && (!empty($requestFromSession[$currentData]))) {
            do_action("eniture_debug_mood", "Session Response (ltl)", json_decode($requestFromSession[$currentData]));
            isset(json_decode($requestFromSession[$currentData])->InstorPickupLocalDelivery) && !empty(json_decode($requestFromSession[$currentData])->InstorPickupLocalDelivery) ? $this->localdeliver = json_decode($requestFromSession[$currentData])->InstorPickupLocalDelivery : $this->localdeliver = array();
            return $this->parse_estes_output($requestFromSession[$currentData], $request_data);
        }

        if (is_array($request_data) && count($request_data) > 0) {
            $estes_curl_obj = new ESTES_Curl_Request();
            $output = $estes_curl_obj->estes_get_curl_response(ESTES_FREIGHT_DOMAIN_HITTING_URL . '/index.php', $request_data);
            // Set response in session
            $response = json_decode($output);

            do_action("eniture_debug_mood", "Response (ltl)", $response);

            isset($response->InstorPickupLocalDelivery) && !empty($response->InstorPickupLocalDelivery) ? $this->localdeliver = $response->InstorPickupLocalDelivery : $this->localdeliver = array();
            if (isset($response->q) &&
                (empty($response->error))) {
                $ratprice = [];
                if (isset($response->q->soapenvBody->ratrateQuote->ratquote->ratpricing->ratprice)) {
                    $ratprice = ((!is_array($response->q->soapenvBody->ratrateQuote->ratquote->ratpricing->ratprice)) ? array($response->q->soapenvBody->ratrateQuote->ratquote->ratpricing->ratprice) : $response->q->soapenvBody->ratrateQuote->ratquote->ratpricing->ratprice);
                }

                if (isset($response->autoResidentialSubscriptionExpired) &&
                    ($response->autoResidentialSubscriptionExpired == 1)) {
                    $flag_api_response = "no";
                    $srequest_data['residential_detecion_flag'] = $flag_api_response;
                    $currentData = md5(json_encode($srequest_data));
                }

                if (is_array($ratprice) && count($ratprice) > 0) {

                    $requestFromSession[$currentData] = $output;
                    WC()->session->set('previousRequestData', $requestFromSession);
                }
            }

            return $this->parse_estes_output($output, $request_data);
        }
    }

    /**
     * @param $label_sufex
     * @return array|bool
     */
    public function label_R_freight_view($label_sufex)
    {
        if ($this->quote_settings['estes_setting_residential'] == 'yes' && (in_array("R", $label_sufex))) {
            $label_sufex = array_flip($label_sufex);

            unset($label_sufex['R']);
            $label_sufex = array_keys($label_sufex);
        }

        if (isset($this->quote_settings['inside_delivery']) && $this->quote_settings['inside_delivery'] == 'yes' && (in_array("INS", $label_sufex))) {
            $label_sufex = array_flip($label_sufex);

            unset($label_sufex['INS']);
            $label_sufex = array_keys($label_sufex);
        }

        return $label_sufex;
    }

    /**
     * @return array
     */
    function estes_freight_service_options_array()
    {
        $active_services = array();
        if (get_option('ltl_standard_transit') == 'yes') {
            $active_services[] = '99';
        }

        return $active_services;
    }

    /**
     * Get Shipping Array For Single Shipment
     * @param $output
     * @return string
     */
    function parse_estes_output($output, $request_data)
    {
        $result = json_decode($output);
        $liftgate_as_option = new Estes_Quotes_Liftgate_As_Option();

        // FDO
        $en_fdo_meta_data = (isset($request_data['en_fdo_meta_data'])) ? $request_data['en_fdo_meta_data'] : '';
        if (isset($result->fdo_handling_unit)) {
            $en_fdo_meta_data['handling_unit_details'] = $result->fdo_handling_unit;
        }

        $accessorials = [];

        // White Glove
        (isset($this->quote_settings['inside_delivery']) && $this->quote_settings['inside_delivery'] == "yes") ? $accessorials[] = "INS" : "";
        ($this->quote_settings['liftgate_delivery'] == "yes") ? $accessorials[] = "L" : "";
        ($this->quote_settings['estes_setting_residential'] == "yes") ? $accessorials[] = "R" : "";
        (isset($request_data['hazardous']) && is_array($request_data['hazardous']) && !empty($request_data['hazardous'])) ? $accessorials[] = "H" : "";

        // Standard Packaging
        $standard_packaging = isset($result->standardPackagingData) ? $result->standardPackagingData : [];

        $label_sufex_arr = $liftgate_as_option->filter_label_sufex_array_estes_quotes($result);

        if ($this->quote_settings['liftgate_delivery'] == "yes") {
            foreach ($label_sufex_arr as $key => $label_sufix) {
                if ($label_sufix == 'L') {
                    unset($label_sufex_arr[$key]);
                }
            }
        }

        $quotes_price_sorted_key = array();
        $simple_quotes_price_sorted_key = array();

        if (isset($result->q) && empty($result->q->Error)) {

            $count = 0;

            foreach ($result->q as $key => $quote) {

                if ($quote->ratserviceLevel->ratid == 99) {

                    // Cuttoff Time
                    $delivery_estimates = (isset($quote->ratdelivery->totalTransitTimeInDays)) ? $quote->ratdelivery->totalTransitTimeInDays : '';
                    $delivery_time_stamp = (isset($quote->ratdelivery->ratdate)) ? $quote->ratdelivery->ratdate : '';

                    $price = (isset($quote->ratpricing->rattotalPrice)) ? $quote->ratpricing->rattotalPrice : 0;
                    $estes_Freight_Shipping = new ESTES_Shipping_Class();

                    // Add product level markup
                    if (!empty($request_data['product_level_markup'])) {
                        $price = $estes_Freight_Shipping->add_estes_handling_fee($price, $request_data['product_level_markup']);
                    }
    
                    // Add origin level markup
                    if (!empty($request_data['origin_markup'])) {
                        $price = $estes_Freight_Shipping->add_estes_handling_fee($price, $request_data['origin_markup']);
                    }

                    $transit = (isset($quote->ratdelivery->ratdate)) ? $quote->ratdelivery->ratdate : '';
                    $deliveryDate = (isset($quote->ratpickup->ratdate)) ? $quote->ratpickup->ratdate : '';
                    $title = (isset($quote->ratserviceLevel->rattext)) ? $quote->ratserviceLevel->rattext : '';
                    $meta_data['service_type'] = $title;
                    $meta_data['accessorials'] = json_encode($accessorials);
                    $meta_data['sender_origin'] = $request_data['sender_origin'];
                    $meta_data['product_name'] = json_encode($request_data['product_name']);

                    // Standard Packaging
                    $meta_data['standard_packaging'] = wp_json_encode($standard_packaging);
                    $surcharge_obj = (isset($quote->rataccessorialInfo->rataccessorial)) ? $quote->rataccessorialInfo->rataccessorial : '';
                    $quotes[$count] = array(
                        'id' => $meta_data['service_type'],
                        'cost' => $price,
                        'label' => (strlen($this->quote_settings['label']) > 0 ? $this->quote_settings['label'] : 'Freight'),
                        'transit_time' => $transit,
                        'transit_delivery_time' => $deliveryDate,
                        // Cuttoff Time
                        'delivery_estimates' => $delivery_estimates,
                        'delivery_time_stamp' => $delivery_time_stamp,
                        'label_sfx_arr' => $label_sufex_arr,
                        'surcharges' => (isset($quote->ratpricing)) ? $this->update_parse_estes_quotes_output($surcharge_obj) : 0,
                        'meta_data' => $meta_data,
                        'markup' => $this->quote_settings['handling_fee'],
                        'origin_markup' => $request_data['origin_markup'],
                        'product_level_markup' => $request_data['product_level_markup'],
                        'plugin_name' => 'daylight',
                        'plugin_type' => 'ltl',
                        'owned_by' => 'eniture',
                    );

                    // FDO
                    $en_fdo_meta_data['rate'] = $quotes[$count];
                    if (isset($en_fdo_meta_data['rate']['meta_data'])) {
                        unset($en_fdo_meta_data['rate']['meta_data']);
                    }
                    $en_fdo_meta_data['quote_settings'] = $this->quote_settings;
                    $quotes[$count]['meta_data']['en_fdo_meta_data'] = $en_fdo_meta_data;

                    $quotes_price_sorted_key[$count] = (isset($quotes[$count]['cost'])) ? $quotes[$count]['cost'] : 0;

                    $quotes[$count] = apply_filters("en_woo_addons_web_quotes", $quotes[$count], en_woo_plugin_estes_ltl_quotes);
                    $label_sufex = (isset($quotes[$count]['label_sfx_arr'])) ? $quotes[$count]['label_sfx_arr'] : [];

                    // White Glove
                    $surcharges = isset($quotes[$count]['surcharges']) ? $quotes[$count]['surcharges'] : [];
                    if (in_array('INS', $accessorials) || (in_array('Y', $label_sufex)) || (isset($surcharges['INS']))) {
                        $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['inside'] = true;
                        $label_sufex[] = 'INS';
                    }

                    $label_sufex = $this->label_R_freight_view($label_sufex);

                    $quotes[$count]['label_sufex'] = $label_sufex;

                    in_array('R', $label_sufex_arr) ? $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['residential'] = true : '';
                    ($this->quote_settings['liftgate_resid_delivery'] == "yes") && (in_array("R", $label_sufex)) && in_array('L', $label_sufex_arr) ? $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['liftgate'] = true : '';

                    if (($this->quote_settings['liftgate_delivery_option'] == "yes") &&
                        (($this->quote_settings['liftgate_resid_delivery'] == "yes") && (!in_array("R", $label_sufex)) ||
                            ($this->quote_settings['liftgate_resid_delivery'] != "yes"))
                    ) {

                        $service = $quotes[$count];
                        $quotes[$count]['id'] .= " WL";

                        if (isset($quotes[$count]['label_sufex']) && (!empty($quotes[$count]['label_sufex']))) {
                            array_push($quotes[$count]['label_sufex'], "L");
                        } else {
                            $quotes[$count]['label_sufex'] = array("L");
                        }

                        // FDO
                        $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['liftgate'] = true;

                        $liftgate_charge = (isset($service['surcharges']['LGATE'])) ? $service['surcharges']['LGATE'] : 0;
                        $service['cost'] = (isset($service['cost'])) ? $service['cost'] - $liftgate_charge : 0;
                        $service_label_sufex = [];
                        (!empty($service)) && (in_array("R", $service['label_sufex'])) ? $service_label_sufex[] = 'R' : '';
                        (!empty($service)) && (in_array("INS", $service['label_sufex'])) ? $service_label_sufex[] = 'INS' : '';
                        $service['label_sufex'] = $service_label_sufex;

                        $simple_quotes[$count] = $service;

                        // FDO
                        if (isset($simple_quotes[$count]['meta_data']['en_fdo_meta_data']['rate']['cost'])) {
                            $simple_quotes[$count]['meta_data']['en_fdo_meta_data']['rate']['cost'] = $service['cost'];
                        }

                        $simple_quotes_price_sorted_key[$count] = (isset($simple_quotes[$count]['cost'])) ? $simple_quotes[$count]['cost'] : 0;
                    }
                }

                $count++;
            }

            // When Hold At Terminal Enabled
            $hold_at_terminal = apply_filters('estes_ltl_quotes_quotes_plans_suscription_and_features', 'estes_freight_hold_at_terminal');
            if (isset($result->holdAtTerminalResponse) && !is_array($hold_at_terminal) && $this->quote_settings['HAT_status'] == 'yes' || (isset($result->holdAtTerminalResponse->severity) && $result->holdAtTerminalResponse->severity != 'ERROR')) {

                foreach ($result->holdAtTerminalResponse->q as $key => $quote) {

                    if ($quote->ratserviceLevel->ratid == 99) {

                        // Cuttoff Time
                        $delivery_estimates = (isset($quote->ratdelivery->totalTransitTimeInDays)) ? $quote->ratdelivery->totalTransitTimeInDays : '';
                        $delivery_time_stamp = (isset($quote->ratdelivery->ratdate)) ? $quote->ratdelivery->ratdate : '';

                        $hold_at_terminal_fee = (isset($quote->ratpricing->rattotalPrice)) ? $quote->ratpricing->rattotalPrice : 0;
                        $transit = $quote->ratdelivery->ratdate;
                        $estes_Freight_Shipping = new ESTES_Shipping_Class();
                        
                        // Add product level markup
                        if (!empty($request_data['product_level_markup'])) {
                            $hold_at_terminal_fee = $estes_Freight_Shipping->add_estes_handling_fee($hold_at_terminal_fee, $request_data['product_level_markup']);
                        }
        
                        // Add origin level markup
                        if (!empty($request_data['origin_markup'])) {
                            $hold_at_terminal_fee = $estes_Freight_Shipping->add_estes_handling_fee($hold_at_terminal_fee, $request_data['origin_markup']);
                        }

                        if (isset($this->quote_settings['HAT_fee']) && (strlen($this->quote_settings['HAT_fee']) > 0)) {
                            $hold_at_terminal_fee = $estes_Freight_Shipping->add_estes_handling_fee($hold_at_terminal_fee, $this->quote_settings['HAT_fee']);
                        }

                        $meta_data['service_type'] = 'FreightHAT';
                        $_accessorials = (in_array('H', $accessorials)) ? array('HAT', 'H') : array('HAT');
                        $meta_data['accessorials'] = json_encode($_accessorials);
                        $meta_data['sender_origin'] = $request_data['sender_origin'];
                        $meta_data['product_name'] = json_encode($request_data['product_name']);
                        $meta_data['address'] = (isset($result->holdAtTerminalResponse->terminalInfo->address)) ? json_encode($result->holdAtTerminalResponse->terminalInfo->address) : array();
                        $meta_data['_address'] = (isset($result->holdAtTerminalResponse->terminalInfo->address, $result->holdAtTerminalResponse->terminalInfo->phoneNumber)) ? $this->get_address_terminal($result->holdAtTerminalResponse->terminalInfo->address, $result->holdAtTerminalResponse->terminalInfo->phoneNumber, '') : '';
                        // Standard packaging
                        $meta_data['standard_packaging'] = wp_json_encode($standard_packaging);

                        $hold_at_terminal_resp = (isset($result->holdAtTerminalResponse)) ? $result->holdAtTerminalResponse : [];

                        $hat_quotes = array(
                            'id' => $meta_data['service_type'],
                            'cost' => $hold_at_terminal_fee,
                            'label' => (strlen($this->quote_settings['label']) > 0) ? $this->quote_settings['label'] : 'Freight',
                            'address' => $meta_data['address'],
                            '_address' => $meta_data['_address'],
                            'transit_time' => $transit,
                            // Cuttoff Time
                            'delivery_estimates' => $delivery_estimates,
                            'delivery_time_stamp' => $delivery_time_stamp,
                            'label_sfx_arr' => $label_sufex_arr,
                            'hat_append_label' => ' with hold at terminal',
                            '_hat_append_label' => $meta_data['_address'],
                            'meta_data' => $meta_data,
                            'markup' => $this->quote_settings['handling_fee'],
                            'origin_markup' => $request_data['origin_markup'],
                            'product_level_markup' => $request_data['product_level_markup'],
                            'plugin_name' => 'daylight',
                            'plugin_type' => 'ltl',
                            'owned_by' => 'eniture',
                        );

                        // -100% Fee is invalid
                        if (isset($this->quote_settings['HAT_fee']) &&
                            ($this->quote_settings['HAT_fee'] == "-100%")) {
                            $hat_quotes = array();
                        }

                        // FDO
                        $en_fdo_meta_data['rate'] = $hat_quotes;
                        if (isset($en_fdo_meta_data['rate']['meta_data'])) {
                            unset($en_fdo_meta_data['rate']['meta_data']);
                        }

                        if (isset($en_fdo_meta_data['rate']['label'], $hat_quotes['hat_append_label'], $hat_quotes['_hat_append_label'])) {
                            $en_fdo_meta_data['rate']['label'] .= $hat_quotes['hat_append_label'] . $hat_quotes['_hat_append_label'];
                        }

                        $en_fdo_meta_data['quote_settings'] = $this->quote_settings;
                        $en_fdo_meta_data['holdatterminal'] = $hold_at_terminal_resp;
                        $hat_quotes['meta_data']['en_fdo_meta_data'] = $en_fdo_meta_data;

                        // FDO
                        $fdo_accessorials = [];
                        if (in_array('H', $accessorials)) {
                            $fdo_accessorials['hazmat'] = true;
                        }

                        $hat_quotes['meta_data']['en_fdo_meta_data']['accessorials'] = $fdo_accessorials;
                    }
                }
            }
        } else {
            return [];
            $count = 0;

            $meta_data['service_type'] = '';
            $meta_data['accessorials'] = json_encode($accessorials);
            $meta_data['sender_origin'] = $request_data['sender_origin'];
            $meta_data['product_name'] = json_encode($request_data['product_name']);
            // Standard packaging
            $meta_data['standard_packaging'] = wp_json_encode($standard_packaging);

            $quotes[$count] = array(
                'id' => 'no_quotes',
                'cost' => 0,
                'label' => '',
                'label_sfx_arr' => [],
                'surcharges' => [],
                'meta_data' => $meta_data,
            );

            // FDO
            $en_fdo_meta_data['rate'] = $quotes[$count];
            if (isset($en_fdo_meta_data['rate']['meta_data'])) {
                unset($en_fdo_meta_data['rate']['meta_data']);
            }
            $en_fdo_meta_data['quote_settings'] = $this->quote_settings;
            $quotes[$count]['meta_data']['en_fdo_meta_data'] = $en_fdo_meta_data;

            $quotes_price_sorted_key[$count] = (isset($quotes[$count]['cost'])) ? $quotes[$count]['cost'] : 0;

            $quotes[$count] = apply_filters("en_woo_addons_web_quotes", $quotes[$count], en_woo_plugin_estes_ltl_quotes);

            $label_sufex = (isset($quotes[$count]['label_sfx_arr'])) ? $quotes[$count]['label_sfx_arr'] : array();

            $label_sufex = $this->label_R_freight_view($label_sufex);

            $quotes[$count]['label_sufex'] = $label_sufex;

            in_array('R', $label_sufex_arr) ? $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['residential'] = true : '';
            ($this->quote_settings['liftgate_resid_delivery'] == "yes") && (in_array("R", $label_sufex)) && in_array('L', $label_sufex_arr) ? $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['liftgate'] = true : '';

            if (($this->quote_settings['liftgate_delivery_option'] == "yes") &&
                (($this->quote_settings['liftgate_resid_delivery'] == "yes") && (!in_array("R", $label_sufex)) ||
                    ($this->quote_settings['liftgate_resid_delivery'] != "yes"))
            ) {

                $service = $quotes[$count];
                $quotes[$count]['id'] .= " WL";

                (isset($quotes[$count]['label_sufex']) &&
                    (!empty($quotes[$count]['label_sufex']))) ?
                    array_push($quotes[$count]['label_sufex'], "L") : // IF
                    $quotes[$count]['label_sufex'] = array("L");       // ELSE
                // FDO
                $quotes[$count]['meta_data']['en_fdo_meta_data']['accessorials']['liftgate'] = true;

                $liftgate_charge = (isset($service['surcharges']['LGATE'])) ? $service['surcharges']['LGATE'] : 0;
                $service['cost'] = (isset($service['cost'])) ? $service['cost'] - $liftgate_charge : 0;
                (!empty($service)) && (in_array("R", $service['label_sufex'])) ? $service['label_sufex'] = array("R") : $service['label_sufex'] = array();

                $simple_quotes[$count] = $service;

                // FDO
                if (isset($simple_quotes[$count]['meta_data']['en_fdo_meta_data']['rate']['cost'])) {
                    $simple_quotes[$count]['meta_data']['en_fdo_meta_data']['rate']['cost'] = $service['cost'];
                }

                $simple_quotes_price_sorted_key[$count] = (isset($simple_quotes[$count]['cost'])) ? $simple_quotes[$count]['cost'] : 0;
            }
        }


//          ===================================== array_multisort =======================================
        (!empty($quotes)) ? array_multisort($quotes_price_sorted_key, SORT_ASC, $quotes) : "";
        (!empty($simple_quotes)) ? array_multisort($simple_quotes_price_sorted_key, SORT_ASC, $simple_quotes) : "";
//          =============================================================================================

        (!empty($simple_quotes)) ? $quotes['simple_quotes'] = $simple_quotes : "";
        (!empty($hat_quotes)) ? $quotes['hold_at_terminal_quotes'] = $hat_quotes : "";

        return $quotes;
    }

    /**
     * Get Terminal Address From Response
     * @param object $address
     * @param string $phone_nbr
     * @param object $distance
     * @return string
     */
    public function get_address_terminal($address, $phone_nbr, $distance)
    {
        $address_terminal = '';
        $phone_nbr_str = '';
        if (!empty($phone_nbr) && (isset($phone_nbr->trancountry, $phone_nbr->tranareaCode, $phone_nbr->transubscriber))) {
            $phone_nbr_str = $phone_nbr->trancountry . ' ' . $phone_nbr->tranareaCode . ' ' . $phone_nbr->transubscriber;
        }

        $address_terminal .= (isset($distance->text)) ? ' | ' . $distance->text : '';
        $address_terminal .= (isset($address->tranline1)) ? ' | ' . $address->tranline1 : '';
        $address_terminal .= (isset($address->trancity)) ? ' ' . $address->trancity : '';
        $address_terminal .= (isset($address->transtateProvince)) ? ' ' . $address->transtateProvince : '';
        $address_terminal .= (isset($address->tranpostalCode)) ? ' ' . $address->tranpostalCode : '';
        $address_terminal .= (strlen($phone_nbr_str) > 0) ? ' | T: ' . $phone_nbr_str : '';
        return $address_terminal;
    }

}
