<?php
/**
 * Plugin Name:    LTL Freight Quotes - Estes Edition
 * Plugin URI:     https://eniture.com/products/
 * Description:    Dynamically retrieves your negotiated shipping rates from Estes Freight and displays the results in the WooCommerce shopping cart.
 * Version:        3.3.1
 * Author:         Eniture Technology
 * Author URI:     http://eniture.com/
 * Text Domain:    eniture-technology
 * License:        GPL version 2 or later - http://www.eniture.com/
 * WC requires at least: 5.7
 * WC tested up to: 8.2.2
 */
/**
 * ESTES Freight
 *
 * @package     ESTES Freight Quotes
 * @author      Eniture-Technology
 */

if (!defined('ABSPATH')) {
    exit;
}

define('ESTES_FREIGHT_DOMAIN_HITTING_URL', 'https://ws031.eniture.com');
define('ESTES_FDO_HITTING_URL', 'https://freightdesk.online/api/updatedWoocomData');
define('ESTES_MAIN_FILE', __FILE__);

add_action('before_woocommerce_init', function () {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    }
});

// Define reference
function en_estes_freight_plugin($plugins)
{
    $plugins['lfq'] = (isset($plugins['lfq'])) ? array_merge($plugins['lfq'], ['estes' => 'ESTES_Shipping_Class']) : ['estes' => 'ESTES_Shipping_Class'];
    return $plugins;
}

add_filter('en_plugins', 'en_estes_freight_plugin');

if (!function_exists('en_woo_plans_notification_PD')) {
    function en_woo_plans_notification_PD($product_detail_options)
    {
        $eniture_plugins_id = 'eniture_plugin_';

        for ($e = 1; $e <= 25; $e++) {
            $settings = get_option($eniture_plugins_id . $e);
            if (isset($settings) && (!empty($settings)) && (is_array($settings))) {
                $plugin_detail = current($settings);
                $plugin_name = (isset($plugin_detail['plugin_name'])) ? $plugin_detail['plugin_name'] : "";

                foreach ($plugin_detail as $key => $value) {
                    if ($key != 'plugin_name') {
                        $action = $value === 1 ? 'enable_plugins' : 'disable_plugins';
                        $product_detail_options[$key][$action] = (isset($product_detail_options[$key][$action]) && strlen($product_detail_options[$key][$action]) > 0) ? ", $plugin_name" : "$plugin_name";
                    }

                }

            }

        }

        return $product_detail_options;
    }

    add_filter('en_woo_plans_notification_action', 'en_woo_plans_notification_PD', 10, 1);
}

if (!function_exists('en_woo_plans_notification_message')) {
    function en_woo_plans_notification_message($enable_plugins, $disable_plugins)
    {
        $enable_plugins = (strlen($enable_plugins) > 0) ? "$enable_plugins: <b> Enabled</b>. " : "";
        $disable_plugins = (strlen($disable_plugins) > 0) ? " $disable_plugins: Upgrade to <b>Standard Plan to enable</b>." : "";
        return $enable_plugins . "<br>" . $disable_plugins;
    }

    add_filter('en_woo_plans_notification_message_action', 'en_woo_plans_notification_message', 10, 2);
}

//Product detail set plans notification message for nested checkbox
if (!function_exists('en_woo_plans_nested_notification_message')) {

    function en_woo_plans_nested_notification_message($enable_plugins, $disable_plugins, $feature)
    {
        $enable_plugins = (strlen($enable_plugins) > 0) ? "$enable_plugins: <b> Enabled</b>. " : "";
        $disable_plugins = (strlen($disable_plugins) > 0 && $feature == 'nested_material') ? " $disable_plugins: Upgrade to <b>Advance Plan to enable</b>." : "";
        return $enable_plugins . "<br>" . $disable_plugins;
    }

    add_filter('en_woo_plans_nested_notification_message_action', 'en_woo_plans_nested_notification_message', 10, 3);
}

/**
 * Load scripts for Estes Freight json tree view
 */
if (!function_exists('en_estes_jtv_script')) {
    function en_estes_jtv_script()
    {
        wp_register_style('en_estes_json_tree_view_style', plugin_dir_url(__FILE__) . 'logs/en-json-tree-view/en-jtv-style.css');
        wp_register_script('en_estes_json_tree_view_script', plugin_dir_url(__FILE__) . 'logs/en-json-tree-view/en-jtv-script.js', ['jquery'], '1.0.0');

        wp_enqueue_style('en_estes_json_tree_view_style');
        wp_enqueue_script('en_estes_json_tree_view_script', [
            'en_tree_view_url' => plugins_url(),
        ]);
    }

    add_action('admin_init', 'en_estes_jtv_script');
}

if (!function_exists('is_plugin_active')) {
    include_once(ABSPATH . 'wp-admin/includes/plugin.php');
}

/**
 * Check WooCommerce installlation
 */
if (!is_plugin_active('woocommerce/woocommerce.php')) {
    add_action('admin_notices', 'estes_wc_avaibility_err');
}
/**
 * Woo Availability Error
 */
function estes_wc_avaibility_err()
{
    $class = "error";
    $message = "LTL Freight Quotes - Estes Edition is enabled, but not effective. It requires WooCommerce in order to work, Please <a target='_blank' href='https://wordpress.org/plugins/woocommerce/installation/'>Install</a> WooCommerce Plugin. Reactivate LTL Freight Quotes - Estes Edition plugin to create LTL shipping class.";
    echo "<div class=\"$class\"> <p>$message</p></div>";
}

/**
 * Check WooCommerce version compatibility
 */

add_action('admin_init', 'estes_check_woo_version');
/**
 * Check Woo Version
 */
function estes_check_woo_version()
{
    $woo_version = estes_wc_version_number();
    $version = '2.6';
    if (!version_compare($woo_version, $version, ">=")) {
        add_action('admin_notices', 'estes_wc_version_failure');
    }
}

/**
 * Version Failure
 */
function estes_wc_version_failure()
{
    ?>
    <div class="notice notice-error">
        <p>
            <?php
            _e('LTL Freight Quotes - Estes Edition plugin requires WooCommerce version 2.6 or higher to work. Functionality may not work properly.', 'wwe-woo-version-failure');
            ?>
        </p>
    </div>
    <?php
}

/**
 * WooCommerce version
 * @return Int | Null
 */
function estes_wc_version_number()
{
    $plugin_folder = get_plugins('/' . 'woocommerce');
    $plugin_file = 'woocommerce.php';

    if (isset($plugin_folder[$plugin_file]['Version'])) {
        return $plugin_folder[$plugin_file]['Version'];
    } else {
        return NULL;
    }
}


if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins'))) || is_plugin_active_for_network('woocommerce/woocommerce.php')) {

    /*
     * Load scripts for Estes
     */

    add_action('admin_enqueue_scripts', 'estes_admin_script');
    /**
     * Admin Scripts
     */
    function estes_admin_script()
    {
        // Cuttoff Time
        wp_register_style('estes_wickedpicker_style', plugin_dir_url(__FILE__) . 'css/wickedpicker.min.css', false, '1.0.0');
        wp_register_script('estes_wickedpicker_script', plugin_dir_url(__FILE__) . 'js/wickedpicker.js', false, '1.0.0');
        wp_enqueue_style('estes_wickedpicker_style');
        wp_enqueue_script('estes_wickedpicker_script');
        wp_register_style('estes-style', plugin_dir_url(__FILE__) . '/css/estes-style.css', false, '1.1.1');
        wp_enqueue_style('estes-style');
    }

    add_action('admin_enqueue_scripts', 'en_estes_script');

    /**
     * Load Front-end scripts for estes
     */
    function en_estes_script()
    {
        wp_enqueue_script('jquery');
        wp_enqueue_script('en_estes_script', plugin_dir_url(__FILE__) . 'js/en-estes.js', array(), '1.1.0');
        wp_localize_script('en_estes_script', 'en_estes_admin_script', array(
            'plugins_url' => plugins_url(),
            'allow_proceed_checkout_eniture' => trim(get_option("allow_proceed_checkout_eniture")),
            'prevent_proceed_checkout_eniture' => trim(get_option("prevent_proceed_checkout_eniture")),
            // Cuttoff Time
            'estes_freight_order_cutoff_time' => get_option("estes_freight_order_cut_off_time"),
        ));
    }

    /*
     * Inlude Plugin Files
     */
    require_once 'order/rates/order-rates.php';
    require_once 'order/en-order-export.php';
    require_once 'order/en-order-widget.php';
    require_once 'fdo/en-fdo.php';
    require_once('estes-liftgate-as-option.php');
    require_once('template/connection-settings.php');
    require_once('template/quote-settings.php');
    // Origin terminal address
    add_action('admin_init', 'estes_update_warehouse');

    require_once('product/en-product-detail.php');
    require_once('product/en-common-product-detail.php');

    require_once 'template/csv-export.php';
    /*
     * link files of plans and warehouse
     */
    require_once('warehouse-dropship/wild-delivery.php');
    require_once('warehouse-dropship/get-distance-request.php');
    require_once('standard-package-addon/standard-package-addon.php');
    require_once 'update-plan.php';

    require_once('estes-test-connection.php');
    require_once('estes-shipping-class.php');
    require_once('db/estes-db.php');
    require_once('estes-admin-filter.php');
    require_once('estes-group-package.php');
    require_once('estes-carrier-service.php');
    require_once('estes-wc-update-change.php');
    require_once('estes-curl-class.php');
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

    require_once 'template/products-nested-options.php';
    /*
     * Estes Freight Activation Hook
     */

    register_activation_hook(__FILE__, 'create_estes_wh_db');
    register_activation_hook(__FILE__, 'estes_class');
    register_activation_hook(__FILE__, 'create_estes_option');
    register_activation_hook(__FILE__, 'old_store_estes_ltl_dropship_status');

    register_activation_hook(__FILE__, 'en_estes_freight_activate_hit_to_update_plan');
    register_deactivation_hook(__FILE__, 'en_estes_freight_deactivate_hit_to_update_plan');


    /**
     * estes plugin update now
     * @param array type $upgrader_object
     * @param array type $options
     */
    function en_estes_update_now()
    {
        $index = 'ltl-freight-quotes-estes-edition/ltl-freight-quotes-estes-edition.php';
        $plugin_info = get_plugins();
        $plugin_version = (isset($plugin_info[$index]['Version'])) ? $plugin_info[$index]['Version'] : '';
        $update_now = get_option('en_estes_update_now');

        if ($update_now != $plugin_version) {
            if (!function_exists('en_estes_freight_activate_hit_to_update_plan')) {
                require_once(__DIR__ . '/update-plan.php');
            }

            create_estes_wh_db();
            estes_class();
            create_estes_option();
            old_store_estes_ltl_dropship_status();
            en_estes_freight_activate_hit_to_update_plan();

            update_option('en_estes_update_now', $plugin_version);
        }
    }

    add_action('init', 'en_estes_update_now');

    /*
     * Estes Action And Filters
     */

    add_action('woocommerce_shipping_init', 'estes_init');
    add_filter('woocommerce_shipping_methods', 'add_estes_ltl');
    add_filter('woocommerce_get_settings_pages', 'estes_shipping_sections');
    add_filter('woocommerce_package_rates', 'estes_hide_shipping', 99);
    add_filter('woocommerce_shipping_calculator_enable_city', '__return_true');


    /*
     * Estes action links
     */

    add_filter('plugin_action_links', 'estes_freight_plugin_action', 10, 5);
    /**
     * Plugin action links
     * @staticvar $plugin
     * @param $actions
     * @param $plugin_file
     */
    function estes_freight_plugin_action($actions, $plugin_file)
    {
        static $plugin;
        if (!isset($plugin))
            $plugin = plugin_basename(__FILE__);
        if ($plugin == $plugin_file) {
            $settings = array('settings' => '<a href="admin.php?page=wc-settings&tab=estes_ltl_quotes">' . __('Settings', 'General') . '</a>');
            $site_link = array('support' => '<a href="https://support.eniture.com/" target="_blank">Support</a>');
            $actions = array_merge($settings, $actions);
            $actions = array_merge($site_link, $actions);
        }
        return $actions;
    }

    /*
    * No Quotes Cart Message
    */

    add_filter('woocommerce_cart_no_shipping_available_html', 'estes_cart_html_message');
    /**
     * Cart message
     */
    function estes_cart_html_message()
    {
        echo "<div><p>There are no shipping methods available. Please double check your address, or contact us if you need any help.</p></div>";
    }

}

define("en_woo_plugin_estes_ltl_quotes", "estes_ltl_quotes");

add_action('wp_enqueue_scripts', 'en_ltl_estes_frontend_checkout_script');
/**
 * Load Frontend scripts for ODFL
 */
function en_ltl_estes_frontend_checkout_script()
{
    wp_enqueue_script('jquery');
    wp_enqueue_script('en_ltl_estes_frontend_checkout_script', plugin_dir_url(__FILE__) . 'front/js/en-estes-checkout.js', array(), '1.0.0');
    wp_localize_script('en_ltl_estes_frontend_checkout_script', 'frontend_script', array(
        'pluginsUrl' => plugins_url(),
    ));
}

/**
 * Get Domain Name
 */
if (!function_exists('estes_freight_get_domain')) {
    function estes_freight_get_domain()
    {
        global $wp;
        $url = home_url($wp->request);
        return getHost($url);
    }
}
if (!function_exists('getHost')) {
    function getHost($url)
    {
        $parseUrl = parse_url(trim($url));
        if (isset($parseUrl['host'])) {
            $host = $parseUrl['host'];
        } else {
            $path = explode('/', $parseUrl['path']);
            $host = $path[0];
        }
        return trim($host);
    }
}
/**
 * Plans Common Hooks
 */
add_filter('estes_ltl_quotes_quotes_plans_suscription_and_features', 'estes_ltl_quotes_quotes_plans_suscription_and_features', 1);

function estes_ltl_quotes_quotes_plans_suscription_and_features($feature)
{
    $package = get_option('estes_freight_package');

    $features = array
    (
        'instore_pickup_local_devlivery' => array('3'),
        'hazardous_material' => array('2', '3'),
        'estes_freight_hold_at_terminal' => array('3'),
        'nested_material' => array('3'),
        'insurance_fee' => array('2', '3'),
        // Cuttoff Time
        'estes_cutt_off_time' => array('2', '3'),
    );
    if (get_option('estes_freight_quotes_store_type') == "1") {
        $features['multi_warehouse'] = array('2', '3');
        $features['multi_dropship'] = array('', '0', '1', '2', '3');
    } else {
        $dropship_status = get_option('en_old_user_dropship_status');
        $warehouse_status = get_option('en_old_user_warehouse_status');


        isset($dropship_status) && ($dropship_status == "0") ? $features['multi_dropship'] = array('', '0', '1', '2', '3') : '';
        isset($warehouse_status) && ($warehouse_status == "0") ? $features['multi_warehouse'] = array('2', '3') : '';

    }

    return (isset($features[$feature]) && (in_array($package, $features[$feature]))) ? TRUE : ((isset($features[$feature])) ? $features[$feature] : '');
}

add_filter('estes_ltl_quotes_plans_notification_link', 'estes_ltl_quotes_plans_notification_link', 1);

function estes_ltl_quotes_plans_notification_link($plans)
{
    $plan = current($plans);
    $plan_to_upgrade = "";
    switch ($plan) {
        case 2:
            $plan_to_upgrade = "<a target='_blank' class='plan_color' href='http://eniture.com/plan/woocommerce-estes-ltl-freight/'>Standard Plan required</a>";
            break;
        case 3:
            $plan_to_upgrade = "<a target='_blank' href='http://eniture.com/plan/woocommerce-estes-ltl-freight/'>Advanced Plan required</a>";
            break;
    }

    return $plan_to_upgrade;
}

/**
 *
 * old customer check dropship / warehouse status on plugin update
 */
function old_store_estes_ltl_dropship_status()
{
    global $wpdb;

//  Check total no. of dropships on plugin updation
    $table_name = $wpdb->prefix . 'warehouse';
    $count_query = "select count(*) from $table_name where location = 'dropship' ";
    $num = $wpdb->get_var($count_query);

    if (get_option('en_old_user_dropship_status') == "0" && get_option('estes_freight_quotes_store_type') == "0") {
        $dropship_status = ($num > 1) ? 1 : 0;

        update_option('en_old_user_dropship_status', "$dropship_status");
    } elseif (get_option('en_old_user_dropship_status') == "" && get_option('estes_freight_quotes_store_type') == "0") {
        $dropship_status = ($num == 1) ? 0 : 1;

        update_option('en_old_user_dropship_status', "$dropship_status");
    }

//  Check total no. of warehouses on plugin updation
    $table_name = $wpdb->prefix . 'warehouse';
    $warehouse_count_query = "select count(*) from $table_name where location = 'warehouse' ";
    $warehouse_num = $wpdb->get_var($warehouse_count_query);

    if (get_option('en_old_user_warehouse_status') == "0" && get_option('estes_freight_quotes_store_type') == "0") {
        $warehouse_status = ($warehouse_num > 1) ? 1 : 0;

        update_option('en_old_user_warehouse_status', "$warehouse_status");
    } elseif (get_option('en_old_user_warehouse_status') == "" && get_option('estes_freight_quotes_store_type') == "0") {
        $warehouse_status = ($warehouse_num == 1) ? 0 : 1;

        update_option('en_old_user_warehouse_status', "$warehouse_status");
    }
}
// fdo va
add_action('wp_ajax_nopriv_estes_fd', 'estes_fd_api');
add_action('wp_ajax_estes_fd', 'estes_fd_api');
/**
 * UPS AJAX Request
 */
function estes_fd_api()
{
    $store_name = estes_freight_get_domain();
    $company_id = $_POST['company_id'];
    $data = [
        'plateform'  => 'wp',
        'store_name' => $store_name,
        'company_id' => $company_id,
        'fd_section' => 'tab=estes_ltl_quotes&section=section-4',
    ];
    if (is_array($data) && count($data) > 0) {
        if($_POST['disconnect'] != 'disconnect') {
            $url =  'https://freightdesk.online/validate-company';
        }else {
            $url = 'https://freightdesk.online/disconnect-woo-connection';
        }
        $response = wp_remote_post($url, [
                'method' => 'POST',
                'timeout' => 60,
                'redirection' => 5,
                'blocking' => true,
                'body' => $data,
            ]
        );
        $response = wp_remote_retrieve_body($response);
    }
    if($_POST['disconnect'] == 'disconnect') {
        $result = json_decode($response);
        if ($result->status == 'SUCCESS') {
            update_option('en_fdo_company_id_status', 0);
        }
    }
    echo $response;
    exit();
}
add_action('rest_api_init', 'en_rest_api_init_status_estes');
function en_rest_api_init_status_estes()
{
    register_rest_route('fdo-company-id', '/update-status', array(
        'methods' => 'POST',
        'callback' => 'en_estes_fdo_data_status',
        'permission_callback' => '__return_true'
    ));
}

/**
 * Update FDO coupon data
 * @param array $request
 * @return array|void
 */
function en_estes_fdo_data_status(WP_REST_Request $request)
{
    $status_data = $request->get_body();
    $status_data_decoded = json_decode($status_data);
    if (isset($status_data_decoded->connection_status)) {
        update_option('en_fdo_company_id_status', $status_data_decoded->connection_status);
        update_option('en_fdo_company_id', $status_data_decoded->fdo_company_id);
    }
    return true;
}